@@ -10,3 +10,8 @@ NULL
 "%||%" <- function(x, y) {
   if (is.null(x)) y else x # nolint: coalesce_linter.
 }
+
+# Backport from R 4.6.0
+`%notin%` <- function(x, table) {
+  match(x, table, nomatch = 0L) == 0
+}
