@@ -100,15 +100,7 @@ read_zarr_array <- function(zarr_array_path, index, s3_client) {
 
   res <- read_data(required_chunks, zarr_array_path, s3_client, index, metadata)
 
-  if (isTRUE(res$warn > 0)) {
-    warning(
-      "Integer overflow detected in at least one chunk.\n",
-      "Overflowing values have been replaced with NA",
-      call. = FALSE
-    )
-  }
-
-  return(res$output)
+  return(res)
 }
 
 .extract_elements <- function(
@@ -150,19 +142,17 @@ read_zarr_array <- function(zarr_array_path, index, s3_client) {
     s3_client = s3_client,
     alt_chunk_dim = alt_chunk_dim
   )
-  warn <- chunk$warning[1]
-  chunk_data <- chunk$chunk_data
 
-  if (!is.null(chunk_data)) {
+  if (!is.null(chunk)) {
     ## extract the required elements from the chunk
-    chunk_data <- R.utils::extract(
-      chunk_data,
+    chunk <- R.utils::extract(
+      chunk,
       indices = index_in_chunk,
       drop = FALSE
     )
   }
 
-  return(list(chunk_data, index_in_result, warning = warn))
+  return(list(chunk, index_in_result))
 }
 
 
@@ -174,8 +164,6 @@ read_data <- function(
   index,
   metadata
 ) {
-  warn <- 0L
-
   ## determine which chunk each of the requests indices belongs to
   # nolint next: undesirable_function_linter.
   chunk_idx <- mapply(
@@ -192,21 +180,32 @@ read_data <- function(
     metadata
   )
 
+  warnings <- list()
   ## hopefully we can eventually do this in parallel
-  chunk_selections <- lapply(
-    seq_along(chunk_names),
-    function(i) {
-      .extract_elements(
-        current_chunk_index = required_chunks[i, ],
-        current_chunk_name = chunk_names[i],
-        metadata = metadata,
-        index = index,
-        zarr_array_path = zarr_array_path,
-        s3_client = s3_client,
-        chunk_idx = chunk_idx
-      )
+  chunk_selections <- withCallingHandlers(
+    lapply(
+      seq_along(chunk_names),
+      function(i) {
+        .extract_elements(
+          current_chunk_index = required_chunks[i, ],
+          current_chunk_name = chunk_names[i],
+          metadata = metadata,
+          index = index,
+          zarr_array_path = zarr_array_path,
+          s3_client = s3_client,
+          chunk_idx = chunk_idx
+        )
+      }
+    ),
+    warning = function(w) {
+      warnings <<- c(warnings, list(w)) # nolint: undesirable_operator_linter.
+      invokeRestart("muffleWarning")
     }
   )
+  for (w in unique(warnings)) {
+    warning(w)
+  }
+
   ## predefine our array to be populated from the read chunks
   output <- array(metadata$fill_value, dim = lengths(index))
 
@@ -222,9 +221,8 @@ read_data <- function(
       )
       eval(parse(text = cmd))
     }
-    warn <- max(warn, chunk_selections[[i]]$warning[1])
   }
-  return(list(output = output, warn = warn))
+  return(output)
 }
 
 find_chunks_needed <- function(metadata, index) {
@@ -274,12 +272,10 @@ get_decompressed_chunk_size <- function(datatype, dimensions) {
 #'    with the fill value from the array metadata.  If `FALSE` (the default),
 #'    missing chunks will return `NULL`.
 #'
-#' @returns A list of length 2.  The entries should be names "chunk_data" and
-#'   "warning". The first is an array containing the decompressed chunk values,
-#'   the second is an integer indicating whether there were any overflow
-#'   warnings generated will reading the chunk into an R datatype.
+#' @returns An array containing the decompressed chunk values.
 #'
 #' @keywords internal
+# nolint next: cyclocomp_linter.
 read_chunk <- function(
   zarr_array_path,
   chunk_name,
@@ -320,21 +316,13 @@ read_chunk <- function(
   if (is.null(compressed_chunk)) {
     if (fill) {
       return(
-        list(
-          "chunk_data" = array(
-            metadata$fill_value,
-            dim = unlist(metadata$chunk_grid$configuration$chunk_shape)
-          ),
-          "warning" = 0L
+        array(
+          metadata$fill_value,
+          dim = unlist(metadata$chunk_grid$configuration$chunk_shape)
         )
       )
     } else {
-      return(
-        list(
-          "chunk_data" = NULL,
-          "warning" = 0L
-        )
-      )
+      return(NULL)
     }
   }
 
@@ -346,67 +334,9 @@ read_chunk <- function(
   }
   decompressed_chunk <- compressed_chunk
 
-  # Bytes -> Array codecs
-  converted_chunk <- .format_chunk(
-    decompressed_chunk,
-    metadata,
-    alt_chunk_dim
-  )
-  # Array -> Array codecs
-  for (codec in metadata$configured_codecs[["array_array"]]) {
-    converted_chunk[[1]] <- do.call(codec, list(converted_chunk[[1]]))
-  }
-
-  return(converted_chunk)
-}
-
-#' Format the decompressed chunk as an array of the correct type
-#'
-#' When a chunk is decompressed it is returned as a vector of raw bytes.  This
-#' function uses the array metadata to select how to convert the bytes into the
-#' final datatype and then converts the resulting output into an array of the
-#' appropriate dimensions, including re-ordering if the original data is in
-#' row-major order.
-#'
-#' @param decompressed_chunk Raw vector holding the decompressed bytes for this
-#'   chunk.
-#' @param metadata List produced by `.read_array_metadata()` holding the contents
-#'   of the `.zarray` file.
-#' @param alt_chunk_dim The dimensions of the array that should be created from
-#'   this chunk.  Normally this will be the same as the chunk shape in
-#'   `metadata`, but when dealing with edge chunks, which may overlap the true
-#'   extent of the array, the returned array should be smaller than the chunk
-#'   shape.
-#'
-#' @returns A list of length 2.  The first element is the formatted chunk data.
-#'   The second is an integer of length 1, indicating if warnings were
-#'   encountered when converting types
-#'
-#'   If "chunk_data" is larger than the space remaining in destination array
-#'   i.e. it contains the overflowing elements, these will be trimmed when the
-#'   chunk is returned to `read_data()`
-#'
-#' @keywords internal
-.format_chunk <- function(decompressed_chunk, metadata, alt_chunk_dim) {
-  datatype <- metadata$datatype
-
-  bytes_codec_config <- metadata$codecs[["bytes"]]$configuration %||%
-    NA_character_
-
-  decompressed_chunk <- codec_bytes_decode(
-    decompressed_chunk,
-    bytes_codec_config,
-    ifelse(
-      # For unicode, nbytes actually is sizeof(int) * nchar
-      metadata$datatype$base_type == "unicode",
-      4L,
-      metadata$datatype$nbytes
-    )
-  )
-
   ## It doesn't seem clear if the on disk chunk will contain the overflow
   ## values or not, so we try both approaches.
-  actual_chunk_size <- length(decompressed_chunk) / datatype$nbytes
+  actual_chunk_size <- length(decompressed_chunk) / metadata$datatype$nbytes
   if (
     !is.null(metadata$codecs[["vlen_utf8"]]) ||
       (actual_chunk_size ==
@@ -417,36 +347,17 @@ read_chunk <- function(
     chunk_dim <- alt_chunk_dim
   }
 
-  if (datatype$base_type == "string") {
-    if (!is.null(metadata$codecs[["vlen_utf8"]])) {
-      converted_chunk <- .format_object(decompressed_chunk, metadata, datatype)
-      dim(converted_chunk[[1]]) <- chunk_dim
-    } else {
-      converted_chunk <- .format_string(decompressed_chunk, datatype)
-      dim(converted_chunk[[1]]) <- chunk_dim
-    }
-  } else if (datatype$base_type == "unicode") {
-    converted_chunk <- .format_unicode(decompressed_chunk, datatype)
-    dim(converted_chunk[[1]]) <- chunk_dim
-  } else {
-    output_type <- switch(
-      datatype$base_type,
-      "bool" = 0L,
-      "int" = 1L,
-      "uint" = 1L,
-      "float" = 2L
-    )
-    converted_chunk <- .Call(
-      "type_convert_chunk",
-      decompressed_chunk,
-      output_type,
-      datatype$nbytes,
-      datatype$is_signed,
-      chunk_dim,
-      PACKAGE = "Rarr"
+  # Bytes -> Array codecs
+  for (codec in metadata$configured_codecs[["array_bytes"]]) {
+    converted_chunk <- do.call(
+      codec,
+      list(decompressed_chunk, chunk_dim, metadata)
     )
   }
+  # Array -> Array codecs
+  for (codec in metadata$configured_codecs[["array_array"]]) {
+    converted_chunk <- do.call(codec, list(converted_chunk))
+  }
 
-  names(converted_chunk) <- c("chunk_data", "warning")
   return(converted_chunk)
 }
