@@ -28,6 +28,32 @@ check_index <- function(index, metadata) {
   return(index)
 }
 
+.create_chunk_names <- function(chunk_indices, metadata) {
+  dim_separator <- metadata$chunk_key_encoding$configuration$separator %||% "/"
+
+  # This is faster than vapply()
+  chunk_names <- as.vector(apply(
+    chunk_indices,
+    1,
+    paste,
+    collapse = dim_separator
+  ))
+
+  if (metadata[["zarr_format"]] == 3) {
+    if (identical(metadata[["shape"]], 1)) {
+      chunk_names <- "c"
+    } else {
+      chunk_names <- paste(
+        "c",
+        chunk_names,
+        sep = dim_separator
+      )
+    }
+  }
+
+  return(chunk_names)
+}
+
 #' Create a string of the form `x[idx[[1]], idx[[2]]] <- y` for an array `x`
 #' where the number of dimensions is variable.
 #'
@@ -95,6 +121,33 @@ check_index <- function(index, metadata) {
   return(datatype)
 }
 
+.parse_datatype_v3 <- function(typestr) {
+  if (is.list(typestr)) {
+    stop(
+      "Only base data types (not extensions) are supported for Zarr v3 arrays for now",
+      call. = FALSE
+    )
+  }
+  datatype <- list()
+
+  datatype$base_type <- gsub("^([[:alpha:]]+).*", "\\1", typestr)
+
+  # FIXME: it's awkward to have to reconvert to integer after the division
+  datatype$nbytes <- as.integer(
+    as.integer(
+      gsub(x = typestr, pattern = "^[^[:digit:]]+", replacement = "")
+    ) /
+      8L
+  )
+  if (is.na(datatype$nbytes)) {
+    datatype$nbytes <- 1L
+  }
+
+  datatype$is_signed <- datatype$base_type != "uint"
+
+  return(datatype)
+}
+
 
 #' Normalize a Zarr array path
 #'
