@@ -197,7 +197,8 @@ write_zarr_array <- function(
   path <- .normalize_array_path(zarr_array_path)
 
   if (storage.mode(x) == "character" && missing(nchar)) {
-    nchar <- max(base::nchar(x))
+    # +1 to add NUL terminator
+    nchar <- max(base::nchar(x)) + 1
   }
 
   create_empty_zarr_array(
@@ -443,7 +444,7 @@ update_zarr_array <- function(zarr_array_path, x, index) {
     chunk_name = chunk_id,
     metadata = metadata_v3,
     fill = TRUE
-  )[["chunk_data"]]
+  )
 
   ## extract the new values from x and insert them into the chunk
   y <- R.utils::extract(x, indices = idx_in_x) # nolint: object_usage_linter.
