@@ -1,6 +1,6 @@
 #include "type_conversion.h"
 
-SEXP type_convert_chunk(SEXP input, SEXP _new_type, SEXP _n_bytes, SEXP _is_signed, SEXP Rdim) {
+SEXP type_convert_chunk(SEXP input, SEXP _new_type, SEXP _n_bytes, SEXP _is_signed) {
 
   void* p_input = RAW(input);
   SEXP output;
@@ -14,12 +14,12 @@ SEXP type_convert_chunk(SEXP input, SEXP _new_type, SEXP _n_bytes, SEXP _is_sign
     output = PROTECT(type_convert_INTEGER(p_input, xlength(input), n_bytes, is_signed));
   } else if (new_type == 2) {
     output = PROTECT(type_convert_REAL(p_input, xlength(input), n_bytes));
+  } else if (new_type == 3) {
+    output = PROTECT(type_convert_STRING(p_input, xlength(input), n_bytes));
   } else {
     error("Unknown data type\n");
   }
 
-  setAttrib(VECTOR_ELT(output, 0), R_DimSymbol, Rdim);
-
   UNPROTECT(1);
   return output;
 }
@@ -27,7 +27,7 @@ SEXP type_convert_chunk(SEXP input, SEXP _new_type, SEXP _n_bytes, SEXP _is_sign
 SEXP type_convert_INTEGER(void *raw_buffer, R_xlen_t length, int n_bytes, int is_signed) {
 
   int *p_data;
-  SEXP output, data, warning;
+  SEXP data;
   R_xlen_t data_length = length / n_bytes;
   R_xlen_t i;
 
@@ -35,11 +35,6 @@ SEXP type_convert_INTEGER(void *raw_buffer, R_xlen_t length, int n_bytes, int is
   data = PROTECT(allocVector(INTSXP, data_length));
   p_data = INTEGER(data);
 
-  // vector to indicate if a warning has been raised
-  warning = PROTECT(allocVector(INTSXP, 1));
-  INTEGER(warning)[0] = 0;
-  int32_t warn = 0;
-
   if(n_bytes == 1) {
     if(is_signed == 1) {
       for (i = 0; i < data_length; i++) {
@@ -71,38 +66,30 @@ SEXP type_convert_INTEGER(void *raw_buffer, R_xlen_t length, int n_bytes, int is
     if(is_signed == 1) {
       memcpy(p_data, raw_buffer, length);
     } else {
-      warn = uint32_to_int32(raw_buffer, data_length, p_data);
-      INTEGER(warning)[0] = warn;
+      uint32_to_int32(raw_buffer, data_length, p_data);
     }
 
   } else if (n_bytes == 8) {
     // for now we convert to 32bit int and overflow values are NA_integer
     int bit64conversion = 0;
     if (bit64conversion == 0) {
-      warn = int64_to_int32(raw_buffer, data_length, p_data, is_signed);
+      int64_to_int32(raw_buffer, data_length, p_data, is_signed);
     }
-    INTEGER(warning)[0] = warn;
   }
 
-  output = PROTECT(allocVector(VECSXP, 2));
-  SET_VECTOR_ELT(output, 0, data);
-  SET_VECTOR_ELT(output, 1, warning);
-
-  UNPROTECT(3);
-  return(output);
+  UNPROTECT(1);
+  return(data);
 }
 
 SEXP type_convert_REAL(void *raw_buffer, R_xlen_t length, int n_bytes) {
 
   R_xlen_t data_length, i;
   double *p_data;
-  SEXP output, data, warning;
+  SEXP data;
 
   data_length  = length / n_bytes;
   data = PROTECT(allocVector(REALSXP, data_length));
   p_data = REAL(data);
-  warning = PROTECT(allocVector(INTSXP, 1));
-  INTEGER(warning)[0] = 0;
 
   if(n_bytes == 2) {
 
@@ -126,34 +113,47 @@ SEXP type_convert_REAL(void *raw_buffer, R_xlen_t length, int n_bytes) {
     error("%d byte floating point values are not currently supported\n", n_bytes);
   }
 
-  output = PROTECT(allocVector(VECSXP, 2));
-  SET_VECTOR_ELT(output, 0, data);
-  SET_VECTOR_ELT(output, 1, warning);
-
-  UNPROTECT(3);
-  return(output);
+  UNPROTECT(1);
+  return(data);
 }
 
 SEXP type_convert_LOGICAL(void *raw_buffer, R_xlen_t length) {
 
   int *p_data;
-  SEXP output, data, warning;
+  SEXP data;
 
   R_xlen_t data_length = length;
 
   data = PROTECT(allocVector(LGLSXP, data_length));
   p_data = LOGICAL(data);
-  warning = PROTECT(allocVector(INTSXP, 1));
-  INTEGER(warning)[0] = 0;
 
   for (int i = 0; i < data_length; i++) {
     p_data[i] = ((int8_t *)raw_buffer)[i];
   }
 
-  output = PROTECT(allocVector(VECSXP, 2));
-  SET_VECTOR_ELT(output, 0, data);
-  SET_VECTOR_ELT(output, 1, warning);
+  UNPROTECT(1);
+  return(data);
+}
 
-  UNPROTECT(3);
-  return(output);
+SEXP type_convert_STRING(void *raw_buffer, R_xlen_t length, int n_bytes) {
+
+  R_xlen_t data_length = length / n_bytes;
+  R_xlen_t i;
+  SEXP data;
+
+  data = PROTECT(allocVector(STRSXP, data_length));
+
+  for (i = 0; i < data_length; i++) {
+    size_t len =  strlen((char *)raw_buffer + i * n_bytes);
+    // Read up to max length or NUL terminator.
+    // We cannot do one without the other as strings may not be NUL terminated (truncated) and
+    // mkCharLenCE complains about NUL characters in the string.
+    if (len > n_bytes)
+      SET_STRING_ELT(data, i, mkCharLenCE((char *)raw_buffer + i * n_bytes, n_bytes, CE_BYTES));
+    else 
+      SET_STRING_ELT(data, i, mkCharCE((char *)raw_buffer + i * n_bytes, CE_BYTES));
+  }
+
+  UNPROTECT(1);
+  return(data);
 }
