@@ -34,3 +34,25 @@ test_that("float64 zarr arrays can be read correctly", {
   # First column should be all 10.52
   expect_identical(column_major[, 1, ], rep(10.52, 30))
 })
+
+test_that("v2 and v3 return identical results", {
+  zarr_v2 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "float64.zarr",
+    package = "Rarr"
+  )
+  zarr_v3 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "float64_v3.zarr",
+    package = "Rarr"
+  )
+
+  expect_no_condition(f64_v2 <- read_zarr_array(zarr_v2))
+  expect_no_condition(f64_v3 <- read_zarr_array(zarr_v3))
+
+  expect_identical(f64_v2, f64_v3)
+})
